﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using VeterinaryContracts.BusinessLogicContracts;
using VeterinaryContracts.BindingModels;
using VeterinaryContracts.SearchModels;
using VeterinaryContracts.StorageContracts;
using VeterinaryContracts.ViewModels;

namespace VeterinaryBusinessLogic.BusinessLogic
{
	public class ServiceLogic : IServiceLogic
	{
		private readonly ILogger _logger;
		private readonly IServiceStorage _serviceStorage;
		public ServiceLogic(ILogger<ServiceLogic> logger, IServiceStorage serviceStorage)
		{
			_logger = logger;
			_serviceStorage = serviceStorage;
		}
		public List<ServiceViewModel>? ReadList(ServiceSearchModel? model)
		{
			_logger.LogInformation("ReadList. ServiceName:{ServiceName}.Id:{ Id}", model?.ServiceName, model?.Id);
			var list = model == null ? _serviceStorage.GetFullList() : _serviceStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count:{Count}", list.Count);
			return list;
		}
		public ServiceViewModel? ReadElement(ServiceSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. ServiceName:{ServiceName}.Id:{ Id}", model.ServiceName, model.Id);
			var element = _serviceStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
			return element;
		}
		public bool Create(ServiceBindingModel model)
		{
			CheckModel(model);
			if (_serviceStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(ServiceBindingModel model)
		{
			CheckModel(model);
			if (_serviceStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(ServiceBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_serviceStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(ServiceBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.ServiceName))
			{
				throw new ArgumentNullException("Нет названия услуги", nameof(model.ServiceName));
			}
			_logger.LogInformation("Service. ServiceName:{ServiceName} Id: { Id}", model.ServiceName, model.Id);
			var element = _serviceStorage.GetElement(new ServiceSearchModel
			{
				ServiceName = model.ServiceName
			});
			
		}
	}
}
